#!/usr/bin/perl
package UserAgents;
use Data::Dumper;
# Parsing user-agents is a mess. Some times the first word should be considered
# some times the last one some times in the middle.
# This module tries to identify the user-agents with a set of regexps
# 
# Usage:
# Use the identify_ua(string) methods returns an array of the user-agent with
# the version

sub identify_ua{
    my $ua = shift;
    my $browser=undef;
    # Compress space sequences
    $ua=~s/\s+/ /g;
    # Replace / with a space
    $ua=~s/\// /g;
    # Replace ; with a space
    $ua=~s/;/ /g;
    # This returns the first word that is followed with a version 
    if (defined($browser = &id_firefox($ua))){
        return $browser;
    }
    if (defined($browser = &id_opera($ua))){
        return $browser;
    }
    if (defined($browser = &id_safari($ua))){
        return $browser;
    }
    if (defined($browser = &id_msie($ua))){
        return $browser;
    }
    if (defined($browser = &id_msie2($ua))){
        return $browser;
    } 
    my $browser = &first_word($ua);
    if (defined($browser=&first_word($ua))){
        return $browser;
    }
     
    # Nothing found
    return {product=>$ua};
}


# Splits the user-agent string in words; Go through the words keep track of the 
# words until the first word matches a version string and then returns
sub first_word{
    my $ua = shift;
    my @t = split(' ',$ua);
    my $product = "";
    my $version = undef;
    foreach my $elem (@t){
        if ($elem=~/([0-9\._]+)/){
            $version=$1;
            last;
        }else{
            $product.= $elem." ";
        }
    }
    if (defined($version)){
        # Remove leading spaces
        $product=~s/\s+$//g;
        return {product=>$product,version=>$version};
    }
    # Nothing 
    return undef;
}
sub id_firefox{
    my $ua = shift;
    if ($ua=~/Firefox[\/ ]([0-9\.]+)/i){
        return {product=>"Firefox", version=>$1};
    }
    return undef;
}

sub id_msie{
    my $ua = shift;
    if ($ua=~/MSIE[\/ ]([0-9\.]+)/i){
        return  {product=>"Microsoft Internet Explorer",version=>$1};
    }
    return undef;
}

sub id_opera{
    my $ua = shift;
    if ($ua=~/Opera[\/ ]([0-9\.]+)/i){
        return  {product=>"Opera",version=>$1};
    }
    return undef;
}


sub id_safari{
    my $ua = shift;
    if ($ua=~/Safari[\/ ]([0-9\.]+)/i){
        return  {product=>"Safari",version=>$1};
    }
    return undef;
}

sub id_msie2{
    my $ua = shift;
    if ($ua=~/Microsoft Internet Explorer[\/ ]([0-9\.]+)/i){
        return  {product=>"Microsoft Internet Explorer",version=>$1};
    }
    return undef;
}
1;
