#!/usr/bin/perl
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

# Global variables

my $isHelp = 0;
my $database = undef;
my $rootdir = undef;
my $histodata={};
sub usage{
    my $exitcode = shift;
    print <<END
Count different tokens from a captured cookies

OPTIONS
    
    -h Shows this screen
    -d Specifies the sqlite3 database
    -r Specifies the root directory that contains multiple sqlite3 files

AUTHOR

    Gerard Wagener

LICENSE
 
    GPL
END
}

sub inspect_cookie{
    my $cookie = shift;
    my @t = split(';',$cookie);
    foreach my $line (@t){
        my ($key,$value) = split('=',$line);
        # Remove spaces
        $key=~s/\s//g;
        $histodata->{$key}++;
    }
}
sub inspect_database{
    my $database = shift;
    my $dbargs = {AutoCommit => 1, PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or 
        die "Cannot connect  to database $database\n";
    my $query="SELECT cookie from httpreq;"; 
    my $sth = $dbh->prepare($query) or die $dbh->errstr." ($database)\n";
    $sth->execute(); 
    my $cookie;
    $sth->bind_columns(\$cookie);
    while ($sth->fetch()){
        &inspect_cookie($cookie);
    }
    $dbh->disconnect;
}

GetOptions("help"=>\$isHelp, "root=s"=>\$rootdir, 
           "database=s"=>\$database) or exit 1;

if ($isHelp){
    &usage (0);
    exit 0;
}

# Is there only one database
if (defined($database)){
    &inspect_database($database);
}

if (defined($rootdir)){
    opendir H, "$rootdir" or die $?."\n";
    my @files  = readdir(H);
    foreach my $file (@files){
        if ($file=~/\.db$/){
            my $f = "$rootdir/$file";
            print STDERR "Processing $f...\n";
            &inspect_database($f);
        }
    }
    close H;
}

foreach my $x (sort {$histodata->{$b}<=>$histodata->{$a}}keys %{$histodata}){
    my $cnt = $histodata->{$x};
    print "$x $cnt\n";
}
exit 0;
