#!/bin/bash
function usage {
   cat <<EOF
Extract from the tcpick - log database the requests per time slot. By default 
all requests are considered and the requests per second are considered.

OPTIONS
    
    -h Shows this screen
    -w Focus only on www requests (port 80)
    -d Specifies the database file
    -s Requests are grouped in hours

AUTHOR

    Gerard Wagener
EOF
 
    exit $1
}

WWW=
DATABASE=
HOUR=
while getopts "hd:ws" OPTION
do
    case $OPTION in
        h)
            usage 0
            ;;
        d)
            DATABASE=$OPTARG
            ;;
        w)  
            WWW=1
            ;;
        s)  
            HOUR=1
            ;;
        ?)
            usage 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
    echo "A database file must be specified"
    exit 1
fi

QUERYEXT=""
if [ ! -z "$WWW" ]; then
    QUERYEXT=" AND dp=80 ";
fi  

if [ -z "$HOUR" ]; then
    QUERY="select  strftime(\"%s\",timestamp), count(*)  from tcpick where \
           state=\"ESTABLISHED\" $QUERYEXT group by timestamp;"
    echo $QUERY | sqlite3 $DATABASE | tr '|' ' ' || exit 1
else
    QUERY="SELECT STRFTIME(\"%s\",timestamp), COUNT(*) FROM tcpick WHERE \
           state=\"ESTABLISHED\" $QUERYEXT GROUP BY  \
           STRFTIME(\"%Y-%m-%d-%H\",timestamp);"
    echo $QUERY | sqlite3 $DATABASE | tr '|' ' ' || exit 1
fi
exit 0


