#!/bin/bash
TCPICK=/home/gerard/build/bin/tcpick
function usage {
    cat <<EOF
Extract TCP sessions from a directory of pcap files and put them in a directory.

    USAGE 

        $0 [-h] -t target  -s source

OPTIONS
    
    -h  Shows this screen
    -t  Specifies the directory that is going to contain the TCP sessions
    -s  Specifies the directory containing the PCAP files

LICENSE

    GPL

AUTHOR
    
    Gerard Wagener
EOF
    exit $1
}

#User parameters
TARGET=
SOURCE=
# Parse command line options
while getopts "ht:s:" OPTION
do
    case $OPTION in 
        h)
            usage 0
            ;;
        s)
            SOURCE=$OPTARG
            ;;
        t)
            TARGET=$OPTARG
            ;;
        ?) 
            usage 1
            ;;
    esac    
done

# Check mandatory parameters
[ -z "$SOURCE" ] && usage 1
[ -z "$TARGET" ] && usage 1
   
if [ ! -d "$SOURCE" ]; then
    echo "An invalid source directory was  specified"
    exit 1
fi

if [ ! -d  "$TARGET" ]; then
    echo "An invalid target directory was specified"
    exit 1
fi
#Extract absolute path
if [ "$(echo $SOURCE | head -c 1)" != "/" ]; then
    SOURCE="$(pwd)/$SOURCE"
fi

WD=$(pwd)
for i in $(ls $SOURCE); do
    mkdir "$TARGET/$i" || exit 1
    cd "$TARGET/$i" || exit 1
    echo "Processing $SOURCE/$i"
    $TCPICK  -n -td -wR -r "$SOURCE/$i" | gzip -c  > "$TARGET/$i.log.gz" || exit 1
    cd $WD || exit 1
done

