#!/bin/bash
function usage {
   cat <<EOF
Do some general statistics on the log file of tcpick
OPTIONS
    
    -h Shows this screen
    -d Specifies the sqlite3 database 
AUTHOR

    Gerard Wagener
EOF
 
    exit $1
}

DATABASE=
while getopts "hd:" OPTION
do
    case $OPTION in
        h)
            usage 0
            ;;
        d)
            DATABASE=$OPTARG
            ;;
        ?)
            usage 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
    echo "A database file must be specified"
    exit 1
fi

echo "*** General statistics on tcpick log files ***"

echo -n "Minimum timestamp: "
QUERY="SELECT MIN(timestamp) FROM tcpick;";
echo $QUERY | sqlite3 $DATABASE | tr '|' ' ' || exit 1

echo -n "Maximum timestamp: "
QUERY="SELECT MAX(timestamp) FROM tcpick;";
echo $QUERY | sqlite3 $DATABASE | tr '|' ' ' || exit 1


echo -n "Number of TCP packets: "
QUERY="SELECT count(*) FROM tcpick;"
echo $QUERY | sqlite3 $DATABASE | tr '|' ' ' || exit 1

echo -n "Number of established sessions: "
QUERY="SELECT COUNT(*) FROM tcpick WHERE state=\"ESTABLISHED\";"
echo $QUERY | sqlite3 $DATABASE | tr '|' ' ' || exit 1
echo "All statistics done."
exit 0

