#!/bin/bash
# Exit on all errors
set -e
function usage {
   cat <<EOF
Shows HTTP traffic on ports different than 80 and 8080

OPTIONS
    
    -d Specifies the database file
    -h Shows this screen
AUTHOR

    Gerard Wagener
EOF
    exit $1
}
DATABASE=
while getopts "hd:" OPTION
do
    case $OPTION in
        h)
            usage 0
            ;;
        d)
            DATABASE=$OPTARG
            ;;
        ?)
            usage 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
    echo "A database file must be specified"
    exit 1
fi

QUERY="SELECT tcpick.sip, tcpick.dp, httpreq.ua, tcpcontent.tid 
       from httpreq, tcpcontent,tcpick 
       where tcpcontent.filename=httpreq.filename 
       and tcpcontent.tid = tcpick.id 
       and not httpreq.ua=\"\" 
       and not (tcpick.dp=80 or tcpick.dp=8080);";
echo $QUERY | sqlite3 -separator ' ' $DATABASE
exit 0

