#!/usr/bin/perl
use strict;
use Getopt::Long;
use DBI;

my $isHelp = 0;
my $database = undef;
my $binsize = 900;

sub usage{
    my $exitcode = shift;
    print <<END;
Usage: infection rate [-h][-b] -d

Computes the infection rate of vulnerable browsers in a given timeframe

OPTIONS
    
    -h Shows this screen
    -d Specifies the database (works only with one db; see mergedb)
    -b time frame length (default 900 seconds)

AUTHOR
    Gerard Wagener

LICENSE
    GPL
END

    exit $exitcode;
}


GetOptions("help"=>\$isHelp,"binsize=i"=>\$binsize,"database=s"=>\$database)
    or &usage(1);

if ($isHelp){
    &usage(0);
}

if (!defined($database)){
    &usage(1);
}

my $dbargs = {AutoCommit => 0,
              PrintError => 1};
my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs);

#Extract boundaries
#
#Extract minimum timestamp
my $query="SELECT min(strftime(\"%s\",ts)) from tmp_uats;";
my $sth = $dbh->prepare($query) or die;
$sth->execute() or die "";
my $mints = undef;
$sth->bind_columns(\$mints) or die;
$sth->fetch();

#Extract maximum timestamp
my $query="SELECT max(strftime(\"%s\",ts)) as x from tmp_uats;";
my $sth = $dbh->prepare($query) or die;
$sth->execute() or die "";
my $maxts = undef;
$sth->bind_columns(\$maxts) or die;
$sth->fetch();



sub count_browsers{
    my $cts = shift;
    
    # Extract number of non vulnerable browsers
    my $query="SELECT COUNT(*) FROM (SELECT DISTINCT ua FROM tmp_uats WHERE uts < ? AND vul=0);";
    my $sth=$dbh->prepare($query) or die;
    $sth->execute($cts);
    my $nnvul = 0;
    $sth->bind_columns(\$nnvul) or die;
    $sth->fetch();
    
    #Extract number of vulnerable browsers
    $query="SELECT COUNT(*) FROM (SELECT DISTINCT ua FROM tmp_uats WHERE uts < ? AND vul>0);";
    my $sth=$dbh->prepare($query) or die;
    $sth->execute($cts);
    my $nvul = 0;
    $sth->bind_columns(\$nvul) or die;
    $sth->fetch();
    return ($nvul,$nnvul);
}
my $cts = $mints;
# Create bins and count distinct vulnerable browsers and distinct non vulnerable
# browsers. Always cound from beginning 
my $nbrBins=int(($maxts-$mints) /$binsize);

my $i=1; 
do{
    $cts+=$binsize;
    my ($nvul,$nnvul) = &count_browsers($cts);
    my $irate = $nvul/ ($nnvul+$nvul);
    print "$cts $nnvul $nvul $irate\n";
    print STDERR "Processing bin $i of $nbrBins\n";
    $i++;
}while ($cts<$maxts);
