#!/usr/bin/perl
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

# Global variables

my $isHelp = 0;
my $database = undef;
my $rootdir = undef;
my $histodata = {};

sub usage{
    my $exitcode = shift;
    print <<END
Extract from the tcpick - log database the requests per time slot. By default 
all requests are considered and the requests per second are considered.

OPTIONS
    
    -h Shows this screen
    -d Specifies the sqlite3 database
    -r Specifies the root directory that contains multiple sqlite3 files
 
AUTHOR

    Gerard Wagener

LICENSE
 
    GPL
END
}
sub inspect_database{
    my $database = shift;
    my $dbargs = {AutoCommit => 1, PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or 
        die "Cannot connect  to database\n";
    my $query="select dp,count(*) from tcpick where sip=\"213.169.101.2\" and \
           state=\"ESTABLISHED\" group by dp;"; 
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute(); 
    my ($dp, $cnt);
    $sth->bind_columns(\$dp,\$cnt);
    while ($sth->fetch()){
        $histodata->{$dp}+=$cnt;
    }
    $dbh->disconnect;
}

GetOptions("help"=>\$isHelp, "root=s"=>\$rootdir, "database=s"=>\$database)
or exit 1;

if ($isHelp){
    &usage (0);
}

# Is there only one database
if (defined($database)){
    &inspect_database($database);
}

if (defined($rootdir)){
    opendir H, "$rootdir" or die $?."\n";
    my @files  = readdir(H);
    foreach my $file (@files){
        if ($file=~/\.db$/){
            my $f = "$rootdir/$file";
            print STDERR "Processing: $f\n";
            &inspect_database($f);
        }
    }
    close H;
}

# Dump results on stdout
foreach my $dp (sort {$a <=> $b}keys %{$histodata}){
    my $cnt = $histodata->{$dp};
    print "$dp $cnt\n";
}
exit 0;
