#!/usr/bin/perl
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

# Global variables

my $isHelp = 0;
my $database = undef;
my $rootdir = undef;
my $histodata={};
my $interval=1;
my $web = 0;
my $mints=0;

sub usage{
    my $exitcode = shift;
    print <<END
Count tcpsessions per time interval. The timeinterval should be defined
in seconds

OPTIONS
    
    -h Shows this screen
    -d Specifies the sqlite3 database
    -r Specifies the root directory that contains multiple sqlite3 files
    -i interval (default 1 second)
    -w Focus on web traffic
    -m minimum timestamp; that is substracted from all timestamps
 
AUTHOR

    Gerard Wagener

LICENSE
 
    GPL
END
}
sub inspect_database{
    my $database = shift;
    my $dbargs = {AutoCommit => 1, PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or 
        die "Cannot connect  to database $database\n";
    my $qext = "";
    if ($web){
        $qext = " AND dp=80 ";
    }
    my $query="SELECT strftime(\"%s\",timestamp) FROM tcpick WHERE 
               sip=\"213.169.101.2\" AND state=\"ESTABLISHED\" $qext;"; 
    my $sth = $dbh->prepare($query) or die $dbh->errstr." ($database)\n";
    $sth->execute(); 
    my $timestamp;
    $sth->bind_columns(\$timestamp);
    while ($sth->fetch()){
        my $delta =  $timestamp - $mints;
        if ($delta< 0){
            die "Hmm. Looks like a wrong minimum was selected $database\n";
        }
        my $idx=int($delta / $interval);
        $histodata->{$idx}++;
    }
    $dbh->disconnect;
}

GetOptions("help"=>\$isHelp, "root=s"=>\$rootdir, "database=s"=>\$database,
           "min=i"=>\$mints,"web"=>\$web, "interval=i"=>\$interval)
or exit 1;

if ($isHelp){
    &usage (0);
}

# Is there only one database
if (defined($database)){
    &inspect_database($database);
}

if (defined($rootdir)){
    opendir H, "$rootdir" or die $?."\n";
    my @files  = readdir(H);
    foreach my $file (@files){
        if ($file=~/\.db$/){
            my $f = "$rootdir/$file";
            &inspect_database($f);
        }
    }
    close H;
}
# Dump results on stdout
foreach my $inter (sort {$a<=>$b}keys %{$histodata}){
    my $cnt = $histodata->{$inter};
    print "$inter $cnt\n";
}
exit 0;
