#!/bin/bash
SYNCPRG="/data/tor/torgame/scripts/sync-tcpick-log.pl"
function usage {
    cat <<EOF
Synchronize the files of a directory containing the output of tcpick in 
gzip format with an sqlite database 

    USAGE 

        $0 [-h] [-d database] [-r root] -s source

OPTIONS
    
    -h  Shows this screen
    -d  Specifies the sqlite3 database file.
    -s  Specifies the directory containing the tcpick outputs files
    -r  Specifies the root where individual databases per log file should
        be created

LICENSE

    GPL

AUTHOR
    
    Gerard Wagener
EOF
    exit $1
}

#User parameters
DATABASE=
SOURCE=
ROOT=
# Parse command line options
while getopts "hd:s:r:" OPTION
do
    case $OPTION in 
        h)
            usage 0
            ;;
        s)
            SOURCE=$OPTARG
            ;;
        d)
            DATABASE=$OPTARG
            ;;
        r)
            ROOT=$OPTARG
            ;;
        ?) 
            usage 1
            ;;
    esac    
done

# Check mandatory parameters
[ -z "$SOURCE" ] && usage 1

if [ ! -z "$DATABASE" ]; then
    # Does the database already exists? If not create it
    if [ ! -e $DATABASE ]; then
        $SYNCPRG -d $DATABASE -c || exit 1
    fi
else
    if [ -z "$ROOT" ]; then
        echo -n "If no database is set, a root directory for the sqlite files "
        echo "must be\n specified"
        exit 1
    fi
fi

   
if [ ! -d  "$SOURCE" ]; then
    echo "An invalid source directory was specified"
    exit 1
fi

for i in $(ls $SOURCE); do
    if [ ! -z $(echo $SOURCE/$i | grep "gz$") ]; then
        if [ ! -z $DATABASE ]; then
            echo "Processing $i ..."
            zcat $SOURCE/$i | $SYNCPRG -d $DATABASE -s $i || exit 1
        else
            echo "Creating $ROOT/$i.db..."
            $SYNCPRG -c -d "$ROOT/$i.db"
            zcat $SOURCE/$i | $SYNCPRG -d "$ROOT/$i.db" -s $i || exit 1
        fi

    fi    
done
echo "Database $DATABASE was sucessfully synchronized."
exit 0
