#!/usr/bin/perl
$|=101;
# 2009 Gerard Wagener
# License GPL
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

my $dbFile = undef;
my $isHelp= undef;
my $create = undef;

sub help{
    print <<END
Usage: sync-apache [-c] -d database [-h]
Synchronizes an apache log file with an sql lite database

OPTIONS
    -h --help       Shows this screen
    -d --dbfile     Specifies the sqlite database file
    -c --create     Create database and exit

APACHE LOG FORMAT
    LogFormat "%h %l %u %t %{Host}i \"%r\" %>s %b \"%{Referer}i\" 
    \"%{User-Agent}i\""

AUTHOR
    Gerard Wagener

LICENSE 
    GPL
END
}

sub create_database{
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile","","",$dbargs);
    my $query="
            CREATE TABLE apache(
                id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
                rhost VARCHAR(32) NOT NULL, 
                rlgn VARCHAR(32) NOT NULL,
                ruser VARCHAR(64) NOT NULL,
                ts DATETIME NOT NULL,
                host VARCHAR(255) NOT NULL,
                line1 TEXT NOT NULL,
                bs INTEGER NOT NULL,
                refer VARCHAR(255) NOT NULL,
                ua VARCHAR(255) NOT NULL);";            
    print "$query\n";
    $dbh->do($query) or die $dbh->errstr."\n";
    $dbh->commit;
    $dbh->disconnect();
    print "Database was created\n";
    exit(0);
}

# Parse command lines
GetOptions("help"=>\$isHelp,
           "dbfile=s"=>\$dbFile,
           "create"=>\$create
           );

if (defined($isHelp)){
    &help;
    exit(0);
}

if (defined($create)){
    if (!defined($dbFile)){
        die "When you want create a database you need to specify a file";
    }
    &create_database;
}

if (!defined($dbFile)){
    die "An sql lite database file must be specified\n";
}

my $dbargs = {AutoCommit => 0,
              PrintError => 1};
my $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile","","",$dbargs);

sub insert_apache{
    my $query = "INSERT INTO  apache (rhost, rlgn, ruser, ts, host, line1, bs,
                 refer, ua) VALUES (?,?,?,?,?,?,?,?,?);";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute(@_) or die $dbh->errstr."\n";
}

sub parse_date{
    my $date = shift;
    my $h={};
    $h->{'Jan'}='01';
    $h->{'Feb'}='02';
    $h->{'Mar'}='03';
    $h->{'Apr'}='04';
    $h->{'May'}='05';
    $h->{'Jun'}='06';
    $h->{'Jul'}='07';
    $h->{'Aug'}='08';
    $h->{'Sep'}='09';
    $h->{'Oct'}='10';
    $h->{'Nov'}='11';
    $h->{'Dec'}='12';
    if ($date=~s/(\d+)\/(\w+)\/(\d+)://g){
        my ($day, $m, $year) = ($1,$2,$3);
        my $month = $h->{$m};
        if ($date=~s/(\d+):(\d+):(\d+)//g){ 
            my ($hour,$minute,$second) = ($1,$2,$3);
            return "$year-$month-$day $hour:$minute:$second";
        }
    }
    return undef;
}


# Parse apache log file
while (<>){
    my $line = $_;
    chomp($line);
    if ($line=~s/^(\S+) (\S+) (\S+)//g){
        my ($rhost,$rlogname,$ruser) = ($1,$2,$3);
        $line=~s/ \[(.+)\] //g;
        my $ts = parse_date($1);
        if ($line=~s/^(\S+) //g){
            my $host = $1;
            if ($line=~s/^\"([^"]+)\" //g){
                my $firstline=$1;
                if ($line=~s/^(\d+) //g){
                    my $code = $1;
                    if ($line=~s/^(\d+) //g){
                        my $nb = $1;
                        if ($line=~s/\"([^"]+)\" //g){
                            my $refer = $1;
                            if ($line=~s/\"([^"]+)\"//g){
                                my $ua = $1;
                            # Finally this painfull parsing ends ....
                            &insert_apache($rhost, $rlogname,$ruser, $ts, 
                                            $host, $firstline, $nb, $refer,
                                            $ua);  
                            }
                        }
                    }
                }
            }
        }
    }
}
$dbh->commit;
$dbh->disconnect;
exit 0;
