#!/usr/bin/perl
use strict;
use Data::Dumper;
use Getopt::Long;
use DBI;
my $dbFile = undef;
my $isHelp = 0;
my $create = 0;
my $semantic=0;
my $dbh = undef;

my $months = { 
        'Jan'=>'01',
        'Feb'=>'02',
        'Mar'=>'03',
        'Apr'=>'04',
        'May'=>'05',
        'Jun'=>'06',
        'Jul'=>'07',
        'Aug'=>'08',
        'Sep'=>'09',
        'Oct'=>'10',
        'Nov'=>'11',
        'Dec'=>'12',
    };
 sub init_database{
    my $dbargs = { AutoCommit=>0, PrintError=>1};
    $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile", "","",$dbargs)
              or die "Cannot init database\n";
}

sub create_database{
    my $query = "CREATE TABLE proxy (id INTEGER PRIMARY KEY AUTOINCREMENT
                                     NOT NULL,
                                     ts DATETIME NOT NULL,
                                     uri VARCHAR(255), 
                                     ua VARCHAR(255),
                                     fullheaders  VARCHAR(255));";
    print "$query\n";
    $dbh->do($query) or die $dbh->errstr."\n";
    $dbh->commit;
    $dbh->disconnect;
    exit(0);
}

sub usage{
    print <<END;
synch-proxy-log [-c] [-d] [-h]

Synchronize  logs of proxy.pl with an sqlite3 database

    -h Shows this screen
    -c Create tables
    -d Database name
    -s Semantic analysis; heuristic to avoid mis-interpretation between ua and
       fullheaders
END
exit 0;
}
sub conv_month{
    my $m = shift;
    # FIXME these shitty modules Date::Parse do not work? always return undef
    # FIXME Not sure if tshark uses these strings
   if (exists($months->{$m})){
        return $months->{$m};
    } 
    die "Unknown date format\n";
}

#Stupid heuristic, that checks for HTTP methods to see if no HTTP request
#was mixed up with a user-agent. We assume that nobody uses GET as user-agent
#:-).
#@return 1 if it is a http request
#@return 0 otherwise
sub is_http_request{
    my $ua = shift;
    #print "Semantic analysis: $ua\n";
    # Test GET and POST
    if ($ua=~/^GET/){
        return 1;
    }
    if ($ua=~/^POST/){
       return 1;
    }
    #TODO implement other HTTP methods
    return 0;
}

sub insert_record{
    my $query  = undef;
    if (scalar(@_) == 2){
        $query = "INSERT INTO proxy (ts,uri) VALUES (?,?);";
    }
    if (scalar(@_) == 3){
        $query="INSERT INTO proxy (ts,uri,ua) VALUES (?,?,?);";
        #Semantic analysis uri could be an ua if not all fields are set
        if ($semantic and &is_http_request(@_[2])){
            $query="INSERT INTO proxy (ts,uri,fullheaders) VALUES (?,?,?);";
        }
    }
    if (scalar(@_) == 4){
       $query = "INSERT INTO proxy (ts,uri,ua,fullheaders) VALUES (?,?,?,?);";
       if ($semantic and &is_http_request(@_[2])){
            my ($ts,$uri,$ua,$fullheaders) = @_;
            @_=($ts,$uri,$ua." ".$fullheaders);
            $query="INSERT INTO proxy (ts,uri,fullheaders) VALUES (?,?,?);";
       }
    }
    if (defined($query)){
        my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
        $sth->execute(@_) or die $dbh->errstr."\n";
    }
}

sub parse_syslog_info{
    my $line = shift;
    # Replace space groups
    $line=~s/\s+/ /g;
    if ($line=~/^(\w+) (\d+) (\d+):(\d+):(\d+)/){
        my $m = &conv_month($1);
        my $d = sprintf('%02d',$2);
        #FIXME Assume that the experiment is done in 2009
        return "2009-$m-$d $3:$4:$5";
    }
    die "Timestamp could not be parsed\n";
}
GetOptions("help"=>\$isHelp,"dbFile=s"=>\$dbFile, 
           "create"=>\$create,"semantic"=>\$semantic) or die "$_\n";

if ($isHelp){
    &usage;
}
&init_database;

if ($create){
    &create_database;
}

while(<>){
    chomp;
    my $line=$_;
    next unless $line=~/proxy.pl/;
    my ($sysloginfo, $msg) = split(']: ',$line,2);    
    my @t = split('\^I',$msg);
    
    my $ts = parse_syslog_info($sysloginfo);
    unshift(@t,$ts);
    &insert_record(@t);
}
$dbh->commit or die $dbh->errstr."\n";
$dbh->disconnect or die $dbh->errstr."\n";
exit 0;

