#!/usr/bin/perl
$|=101;
# 2009 Gerard Wagener
# License GPL
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

my $dbFile = undef;
my $isHelp= undef;
my $create = undef;
my $srcName = undef;

sub help{
    print <<END
Usage: sync-tcpick [-c] -d database [-h]
Synchronizes a tcppick log file with an sql lite database

OPTIONS
    -h --help       Shows this screen
    -d --dbfile     Specifies the sqlite database file
    -c --create     Create database and exit
    -s --source     Speficies a source name that provided the tcpick logs
                    (Needed to recover content afterwards)

END
}

sub create_database{
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile","","",$dbargs);
    my $query="
CREATE TABLE tcpick(id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
                          timestamp DATETIME, ns INTEGER NOT NULL,
                          idx INTEGER NOT NULL, STATE VARCHAR(32) NOT NULL,
                          SIP VARCHAR(32) NOT NULL, SP INTEGER,
                          DIP VARCHAR(32) NOT NULL, DP INTEGER,
                          filename TEXT); 
                    ); ";
    print "$query\n";
    $dbh->do($query) or die $dbh->errstr."\n";
    $dbh->commit;
    $dbh->disconnect();
    print "Database was created\n";
    exit(0);
}

# Parse command lines
GetOptions("help"=>\$isHelp,
           "dbfile=s"=>\$dbFile,
           "create"=>\$create,
           "source=s"=>\$srcName);

if (defined($isHelp)){
    &help;
    exit(0);
}

if (defined($create)){
    if (!defined($dbFile)){
        die "When you want create a database you need to specify a file";
    }
    &create_database;
}

if (!defined($dbFile)){
    die "An sql lite database file must be specified\n";
}
if (!defined($srcName)){
    die "A source name must be specified for tracking the content\n";
}
my $dbargs = {AutoCommit => 0,
              PrintError => 1};
my $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile","","",$dbargs);

sub insert_tcpick{
    my ($ts, $ns, $idx, $state, $sip, $sp, $dip, $dp) = @_;
    my $query = "INSERT INTO tcpick (timestamp, ns, idx, state, sip, sp,
                 dip,dp,filename) VALUES (?,?,?,?,?,?,?,?,?);";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute($ts,$ns,$idx,$state,$sip,$sp, $dip, $dp,$srcName) 
    or die $dbh->errstr."\n";
}
# Parse tcpick output
while (<>){
    my $line = $_;
    chomp($line);
    # Compress spaces
    $line=~s/\s+/ /g;
    #Take only lines that start with timestamps
    if ($line=~s/^(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)\.(\d+) //g){    
        my ($day,$month,$year,$hour,$minute,$second, $ns) = 
           ($1,$2,$3,$4,$5,$6,$7);
        $line=~s/(\d+) ([^ ]+) ([^:]+):([^ ]+)//g;
        my ($idx, $state, $sip, $sp) = ($1,$2,$3,$4);
        $line=~/(\d+)\.(\d+)\.(\d+)\.(\d+):(.+)/;
        my ($dip,$dp) = ("$1.$2.$3.$4", $5);
        # Create sqlite3 timestamp
        my $ts="20$year-$month-$day $hour:$minute:$second";
        &insert_tcpick($ts,$ns,$idx,$state,$sip,$sp,$dip,$dp);
    }
}
$dbh->commit;
$dbh->disconnect();
exit 0;
