#!/usr/bin/perl
use strict;
use Getopt::Long;
use DBI;
# Global variables
my $isHelp = 0;
my $database = undef;
my $shouldCreate = 0;
my $published = undef;
my $dbh = undef;

sub parse_message{
    my ($msg, $published) = @_;
    my ($r,$s) = split('\n',$msg,2);
    # Consider only exit nodes that are valid
    if ($s=~/Exit/){
        if ($s=~/Valid/){
            $r=~/(\d+\.\d+\.\d+\.\d+)/;
            return $1;
        }
    }
    return undef;
}

sub usage{
    print <<END;
Parse tor-exit status and put the published date and the IP address of a valid
exit node in an sqlite3 database

OPTIONS

    -h  Shows this screen
    -d  Specifies the database
    -c  Create the tables

AUTHOR 
    Gerard Wagener

LICENSE 
    GPL
END
}
sub create_table{
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or
                    die "Cannot connect to $database\n";

    my $query="CREATE TABLE  torstatus(id INTEGER PRIMARY KEY AUTOINCREMENT,
              ts DATETIME, ipaddr VARCHAR(16))";
    $dbh->do($query) or die $dbh->errstr."\n"; 
    $dbh->commit;
    $dbh->disconnect;
    print "*** Table sucessfully created *** \n";
    exit 0;
}

sub insert_record{
    my ($ts,$ipaddr) = @_;
    # If no connection is there connect
    if (!defined($dbh)){
        my $dbargs = {AutoCommit => 0,
                      PrintError => 1};
        $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or
                    die "Cannot connect to $database\n";
    }
    my $query="INSERT into torstatus (ts,ipaddr) VALUES (?,?);";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute($ts,$ipaddr) or die $dbh->errstr."\n";

    
}
# Detect messages parse the content of the URL XXX
sub parse_tor_status{
    my $in_msg = 0;
    my $msg = undef;
    my $published = undef;
    while (<>){
        my $line=$_;
        if ($line=~/^published (.+)/){
            $published = $1;
        }
        if ($line=~/^opt/){
            $in_msg = 1;
            # Dump previous message
            if (defined($msg)){
                my $addr = &parse_message($msg);
                if (defined($published) and defined($addr)){
                    &insert_record($published,$addr);
                }
                # Reset message buffer
                $msg="";
            }
            next;
        }
        if ($in_msg){
            $msg.=$line;
        }
    }
}

GetOptions("help=s"=>\$isHelp, "database=s"=>\$database, 
           "create"=>\$shouldCreate) or exit 1;

if ($isHelp){
    &usage;
    exit 1;
}
if ($shouldCreate){
    &create_table;
    exit 0;
}

# Parse STDIN
&parse_tor_status;
if (defined($dbh)){
    $dbh->commit;
    $dbh->disconnect;
    exit 0;
}

exit 0;
