#!/usr/bin/perl
$|=101;
# 2009 Gerard Wagener
# License GPL
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

my $dbFile = undef;
my $isHelp= undef;
my $create = undef;
my $srcName = undef;

sub help{
    print <<END
Usage: sync-tshark-log [-c] -d database [-h]
Synchronizes the output of tshark with an sql lite database

Currently focuses only on DNS queries
OPTIONS
    -h --help       Shows this screen
    -d --dbfile     Specifies the sqlite database file
    -c --create     Create database and exit

EXAMPLES
    # Create a database dns.db 
    sync-tshark-log -c -d dns.db
    # Populate the database dns.db with information from tshark
    dns-tshark.sh | sync-tshark-log -d dns.db

AUTHOR
    Gerard Wagener

LICENSE
    GPL

END
}

sub conv_month{
    my $m = shift;
    # FIXME these shitty modules Date::Parse do not work? always return undef
    # FIXME Not sure if tshark uses these strings
    my $t ={ 
        'Jan'=>'01',
        'Feb'=>'02',
        'Mar'=>'03',
        'Apr'=>'04',
        'May'=>'05',
        'Jun'=>'06',
        'Jul'=>'07',
        'Aug'=>'08',
        'Sep'=>'09',
        'Oct'=>'10',
        'Nov'=>'11',
        'Dec'=>'12',
    };
    if (exists($t->{$m})){
        return $t->{$m};
    } 
    die "Unknown date format\n";
}
sub create_database{
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile","","",$dbargs);
    my $query="
CREATE TABLE dns(id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
                          framenumber INTEGER NOT NULL, 
                          framelen INTEGER,
                          caplen INTEGER,
                          ethtype INTEGER,
                          ts DATETIME, 
                          ns INTEGER, 
                          ipchecksum_good INTEGER,
                          ipsrc_host VARCHAR(32), 
                          ipdst_host VARCHAR(32), 
                          ipproto INTEGER,
                          udpchecksum_good INTEGER,
                          udpsrcport INTEGER,
                          udpdstport INTEGER,
                          udplength INTEGER,
                          dnsid INTEGER,
                          dnsflags INTEGER,
                          dnsqryname VARCHAR(255),
                          dnsqrytype INTEGER,
                          dnsqryclass INTEGER ); ";
    print "$query\n";
    $dbh->do($query) or die $dbh->errstr."\n";
    $dbh->commit;
    $dbh->disconnect();
    print "Database was created\n";
    exit(0);
}

# Parse command lines
GetOptions("help"=>\$isHelp,
           "dbfile=s"=>\$dbFile,
           "create"=>\$create);

if (defined($isHelp)){
    &help;
    exit(0);
}

if (defined($create)){
    if (!defined($dbFile)){
        die "When you want create a database you need to specify a file";
    }
    &create_database;
}

if (!defined($dbFile)){
    die "An sql lite database file must be specified\n";
}

my $dbargs = {AutoCommit => 0,
              PrintError => 1};
my $dbh = DBI->connect("dbi:SQLite:dbname=$dbFile","","",$dbargs);

sub insert_record{
    #print Dumper(\@_);
    my $query="INSERT into dns (framenumber, framelen,caplen, ethtype,
                                 ts,ns,ipchecksum_good,ipsrc_host,ipdst_host,
                                 ipproto, udpchecksum_good, udpsrcport,
                                 udpdstport,udplength, dnsid,dnsflags,
                                 dnsqryname, dnsqrytype,dnsqryclass) VALUES 
                                 (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?);";  
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute(@_)  or die $dbh->errstr."\n";
}

# Parse tcpick output
while (<>){
    chomp;
    my $line=$_;
    if ($line=~s/^(\d+)\s(\d+)\s(\d+)\s//g){
        my ($fnum,$len,$caplen) = ($1,$2,$3,$4);
        if ($line=~s/^0x([0-9a-fA-F]+)\s//g){
            my $ethtype=hex($1);
            # Extract timestamp
       if ($line=~s/^(\w+)\s+(\d+)+,\s+(\d+)\s+(\d+):(\d+):(\d+)\.(\d+)\s+//g){
            my $m = &conv_month($1);    
            my $ts = "$3-".sprintf('%02d',$m)."-".sprintf('%02d',$2);
            $ts.=' '.sprintf('%02d',$4).':'.sprintf('%02d',$5).':';
            $ts.=sprintf('%02d',$6); 
            my $ns = $7;
            my ($ipchecksum_good,$ipsrc_host, $ipdst_host,$ipproto,
                $udpchecksum_good,$udpsrcport, $udpdstport, $udplength,
                $dnsid, $dnsflags, $dnsqryname,$dnsqrytype,$dnsqryclass)
               =split(/\s+/,$line);
            # Convert the hex values into decimal values
            $ipproto = hex($ipproto);
            $dnsid=hex($dnsid);
            $dnsflags=hex($dnsflags);
            $dnsqrytype=hex($dnsqrytype); 
            $dnsqryclass=hex($dnsqryclass);
            &insert_record ($fnum,$len,$caplen, $ethtype, $ts, $ns, 
                  $ipchecksum_good, $ipsrc_host,$ipdst_host,$ipproto,
                  $udpchecksum_good, $udpsrcport,$udpdstport, $udplength,
                  $dnsid,$dnsflags, $dnsqryname, $dnsqrytype, $dnsqryclass); 
        }
         }
    }
}

$dbh->commit;
$dbh->disconnect();
exit 0;
