#!/usr/bin/perl
use strict;
use UserAgents;
use Data::Dumper;
use Test::Deep;
use Test::Simple tests=> 14;
### Test 1 
my $agent =  UserAgents::identify_ua("Opera/7.21");
my $VAR1 = {
          'version' => '7.21',
          'product' => 'Opera'
        };

cmp_deeply($VAR1,$agent);
### Test 2
my $agent = UserAgents::identify_ua(undef);
$VAR1 = {
          'product' => undef
        };
cmp_deeply($VAR1,$agent);

$agent = UserAgents::identify_ua("AAA");
$VAR1 = {
          'product' => 'AAA'
        };
cmp_deeply($VAR1,$agent);

### Test 3
my $str="Mozilla/5.0 (X11; U; Linux i686; zh-CN; rv:1.9.0.8) Gecko/2009032711 Ubuntu/8.04 (hardy) Firefox/3.0.8";
my $agent = UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '3.0.8',
          'product' => 'Firefox'
        };
cmp_deeply($agent,$VAR1);

### Test 4
# Test other delimiter
my $str="Mozilla/5.0 (X11; U; Linux i686; zh-CN; rv:1.9.0.8) Gecko/2009032711 Ubuntu/8.04 (hardy) Firefox 3.0.8";
my $agent = UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '3.0.8',
          'product' => 'Firefox'
        };
cmp_deeply($agent,$VAR1);
my $str="Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; AntivirXP08; InfoPath.2; Dealio Toolbar 3.4; .NET CLR 2.0.50727; MEGAUPLOAD 2.0; .NET CLR 3.0.04506.30; .NET CLR 3.0.";

### Test 5
my $agent=UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '7.0',
          'product' => 'Microsoft Internet Explorer'
        };
cmp_deeply($VAR1,$agent);

### Test 6
$str="Opera/9.63 (Windows NT 5.1; U; Opera Now!; zh-cn) Presto/2.1.1";
my $agent=UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '9.63',
          'product' => 'Opera'
        };
cmp_deeply($agent,$VAR1);

### Test 7
$str="Mozilla/5.0 (Macintosh; U; Intel Mac OS X; de-de) AppleWebKit/523.10.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
$agent=UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '523.10',
          'product' => 'Safari'
        };
cmp_deeply($agent,$VAR1);

### Test 8
$str="Microsoft Internet Explorer/8.4 (Windows 4.7; en-US;)";
$agent=UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '8.4',
          'product' => 'Microsoft Internet Explorer'
        };
cmp_deeply($agent,$VAR1);

### Test 9
$str="Azureus 2.5.0.4;Linux;Java 1.6.0_07";
$agent=UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '2.5.0.4',
          'product' => 'Azureus'
        };
cmp_deeply($agent,$VAR1);

### Test 10
$str="Yahoo-Java-SDK/dev";
$agent=UserAgents::identify_ua($str);
$VAR1 = {
          'product' => 'Yahoo-Java-SDK dev'
        };
cmp_deeply($agent,$VAR1);

### Test 11
$str="Java/1.6.0_02";
$agent = UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '1.6.0_02',
          'product' => 'Java'
        };
cmp_deeply($agent,$VAR1);
$str="Adobe Update Manager 5";
$agent = UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '5',
          'product' => 'Adobe Update Manager'
        };
cmp_deeply($agent,$VAR1);
$str="AntiVir-NGUpd/9.0.0.42";
$agent=UserAgents::identify_ua($str);
$VAR1 = {
          'version' => '9.0.0.42',
          'product' => 'AntiVir-NGUpd'
        };
cmp_deeply($agent,$VAR1);
