#!/bin/bash
# Exit on all errors
set -e
function usage {
   cat <<EOF
Do a histogram of taged users
 
OPTIONS
    
    -d Specifies the database file
    -h Shows this screen
AUTHOR

    Gerard Wagener
EOF
    exit $1
}
DATABASE=
while getopts "hd:" OPTION
do
    case $OPTION in
        h)
            usage 0
            ;;
        d)
            DATABASE=$OPTARG
            ;;
        ?)
            usage 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
    echo "A database file must be specified"
    exit 1
fi


# Take minimal timestamp from tcpdump; we launched just before the transparent
# proxy iptable rule. Take the maximal timestamp of the proxy we stopped 
# with the tor-exit node

# Get the minimal timestamp
QUERY="SELECT MIN(timestamp) FROM tcpick;" 
MINTS=$(echo $QUERY | sqlite3 $DATABASE)
 
QUERY="SELECT MAX(ts) FROM proxy;" 
MAXTS=$(echo $QUERY | sqlite3 $DATABASE)

#QUERY="SELECT count(*) as x FROM apache WHERE ts >= \"$MINTS\" AND
#       ts <= \"$MAXTS\" AND (NOT host=\"f.exp.quuxlabs.com\")GROUP BY host;" 

QUERY="SELECT X,count(*) FROM (SELECT count(*) as x FROM apache 
       WHERE ts >= \"$MINTS\" AND ts <= \"$MAXTS\" AND 
       (NOT host=\"f.exp.quuxlabs.com\")GROUP BY host) GROUP BY x;" 

echo "#Occurence of the same unique tag Observed frequencies" 
echo $QUERY | sqlite3 -separator ' '  $DATABASE

QUERY="SELECT COUNT(*) FROM apache WHERE ts >= \"$MINTS\" AND 
       ts <= \"$MAXTS\" AND host = \"f.exp.quuxlabs.com\";"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of queries for the fixed image: $CNT" 

QUERY="SELECT COUNT(*) FROM 
       (SELECT DISTINCT dns.ipsrc_host FROM apache,dns WHERE
       apache.host like \"%exp.quuxlabs.com\" 
       AND NOT apache.host like \"f.exp.quuxlabs.com\" 
       AND dns.dnsqryname=apache.host AND dns.ts >= \"$MINTS\"
       AND dns.ts <= \"$MAXTS\"
       );"
CNT=$(echo $QUERY | sqlite3 -separator '  ' $DATABASE)
echo "Number of different IP addresses that got a uniq tag: $CNT"

QUERY="SELECT COUNT(*) FROM (
       SELECT DISTINCT dns.ipsrc_host  FROM apache,dns WHERE 
       apache.host LIKE \"f.exp.quuxlabs.com\" AND 
       dns.ts >= \"$MINTS\" AND dns.ts <=\"$MAXTS\");"
CNT=$QUERY | sqlite3 $DATABASE
echo "Number of different IP addresses that got the static tag: $CNT"
echo "*** end ***" 
exit 0
