#!/bin/bash
# Exit on all errors
set -e
function usage {
   cat <<EOF
Do some user-agent correlation between the proxy and the web server 

OPTIONS
    
    -d Specifies the database file
    -h Shows this screen
AUTHOR

    Gerard Wagener
EOF
    exit $1
}
DATABASE=
while getopts "hd:" OPTION
do
    case $OPTION in
        h)
            usage 0
            ;;
        d)
            DATABASE=$OPTARG
            ;;
        ?)
            usage 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
    echo "A database file must be specified"
    exit 1
fi


# Take minimal timestamp from tcpdump; we launched just before the transparent
# proxy iptable rule. Take the maximal timestamp of the proxy we stopped 
# with the tor-exit node

# Get the minimal timestamp
QUERY="SELECT MIN(timestamp) FROM tcpick;" 
MINTS=$(echo $QUERY | sqlite3 $DATABASE)
# Get the maximal timestamp
QUERY="SELECT MAX(timestamp) FROM tcpick;" 
MAXTS=$(echo $QUERY | sqlite3 $DATABASE)

echo "Minimal timestamp in TCP capture: $MINTS" 
echo "Maximal timestamp in TCP capture: $MAXTS" 

QUERY="SELECT MIN(ts) FROM proxy;"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Minimum timestamp of the proxy: $CNT"
 
QUERY="SELECT MAX(ts) FROM proxy;" 
MAXTS=$(echo $QUERY | sqlite3 $DATABASE)
echo "Maximum timestamp of the proxy: $MAXTS" 

QUERY="SELECT MIN(ts) FROM dns;" 
TS=$(echo $QUERY | sqlite3 $DATABASE)
echo "Minimum timestamp in the DNS capture: $TS"

QUERY="SELECT MAX(ts) FROM dns;" 
TS=$(echo $QUERY | sqlite3 $DATABASE)
echo "Maximum timestamp in the DNS capture: $TS"
 
QUERY="SELECT COUNT(*) FROM apache;"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of hits on the web server: $CNT"

QUERY="SELECT count(*) FROM apache WHERE ts > \"$MAXTS\";"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of hits in the web server after the shutdown of the proxy: $CNT" 

QUERY="SELECT COUNT(*) FROM apache WHERE ts < \"$MINTS\";"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of hits before the TOR operation: $CNT"
 
QUERY="SELECT count(*) FROM apache WHERE ts >= \"$MINTS\" AND ts <= \"$MAXTS\";" 
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of hits on the web server in the period of the opertion of the exitnode: $CNT" 
echo "Host queries after the max timestamps" 
QUERY="SELECT DISTINCT host FROM apache WHERE ts> \"$MAXTS\" ORDER BY ts;"
echo $QUERY | sqlite3 -separator ' ' $DATABASE 

# Inspect proxy logs
QUERY="SELECT COUNT(*) FROM proxy;"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of hits on the proxy: $CNT"

QUERY="SELECT COUNT(*) FROM proxy WHERE ts >= \"$MINTS\" AND ts <= \"$MAXTS\";"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of hits on the proxy during the operation: $CNT"  


# User-agent analysis" 
QUERY="SELECT COUNT(*) FROM (SELECT DISTINCT ua FROM proxy WHERE ts >=\"$MIN\" \
AND ts <=\"$MAXTS\");" 

CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of distinct ua during the exitnode operation on the proxy: $CNT" 

QUERY="SELECT COUNT(*) FROM (SELECT DISTINCT ua FROM apache WHERE ts>=\"$MIN\" \
AND ts <=\"$MAXTS\");"
CNT=$(echo $QUERY | sqlite3 $DATABASE)
echo "Number of distinct ua during the exitnode operation on apache: $CNT"  
exit 0 
