#!/usr/bin/perl
use Data::Dumper;
use Getopt::Long;
use DBI;
use UserAgents;
use FuzzyCVE;
use strict;

# Global variables

my $isHelp = 0;
my $database = undef;
my $rootdir = undef;
my $histodata={};
my $word=0;
my $freq = 0;
my $update = 0;
my $serial = undef;

sub usage{
    my $exitcode = shift;
    print <<END
Count different user-agents from a capture

OPTIONS
    
    -h Shows this screen
    -d Specifies the sqlite3 database
    -r Specifies the root directory that contains multiple sqlite3 files
    -m Try to match user-agents with the user agent module
    -f Print frequencies
    -u Update tmp_uats table; add the number of vulnerabilities

AUTHOR

    Gerard Wagener

LICENSE
 
    GPL
END
}
sub inspect_database{
    my $database = shift;
    my $dbargs = {AutoCommit => 1, PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or 
        die "Cannot connect  to database $database\n";
    my $query="SELECT ua from httpreq;"; 
    my $sth = $dbh->prepare($query) or die $dbh->errstr." ($database)\n";
    $sth->execute(); 
    my $ua;
    $sth->bind_columns(\$ua);
    while ($sth->fetch()){
        $histodata->{$ua}++;
    }
    $dbh->disconnect;
}

sub load_ser_obj{
    my $fileName = shift;
    open Z,"<$fileName" or die "Cannot open serialized object\n";
    my $buf=undef;
    read(Z,$buf,-s $fileName);
    close Z;
    my $VAR1 = FuzzyCVE->new;
    eval $buf;
    return $VAR1;
}


sub update_database{
    my ($database, $fc) = @_;
    my $dbargs = {AutoCommit => 0, PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or 
        die "Cannot connect  to database $database\n";
    my $query="SELECT id,ua from tmp_uats;"; 
    my $sth = $dbh->prepare($query) or die $dbh->errstr." ($database)\n";
    $sth->execute(); 
    my ($id,$ua);
    $sth->bind_columns(\$id,\$ua);
    while ($sth->fetch()){
        my $res=UserAgents::identify_ua($ua);
        #FIXME If no version is explicity set; all versions are used
        my $nv = $fc->get_product_cve_num($res->{product},$res->{version});
        my $q = "UPDATE tmp_uats set vul=? WHERE id=?;";
        my $s = $dbh->prepare($q) or die $dbh->errstr."\n";
        $s->execute($nv,$id) or die $dbh->errstr."\n"; 
    }
    $dbh->commit;
    $dbh->disconnect;
}
GetOptions("match"=>\$word,"help"=>\$isHelp, "root=s"=>\$rootdir, 
           "database=s"=>\$database,"frequency"=>\$freq,"update"=>\$update,
           "serial=s"=>\$serial) 
           or exit 1;

if ($isHelp){
    &usage (0);
}

# Is there only one database
if (defined($database)){
    if ($update){
        # A serialized FuzzyCVE object must be present
        my $fc = &load_ser_obj($serial);
        &update_database($database,$fc);
    }else{
        &inspect_database($database);
    }
}


if (defined($rootdir)){
    opendir H, "$rootdir" or die $?."\n";
    my @files  = readdir(H);
    foreach my $file (@files){
        if ($file=~/\.db$/){
            my $f = "$rootdir/$file";
            if ($update){
                 my $fc = &load_ser_obj($serial);
                 &update_database($f,$fc);
            }else{
                &inspect_database($f);
            }
        }
    }
    close H;
}
if ($update){
    exit 0;
}
if ($word){
    my $h = {};
    # Aggreate results
    foreach my $ua (sort {$a<=>$b}keys %{$histodata}){
        my $cnt = $histodata->{$ua};
        $ua = UserAgents::identify_ua($ua);
        my $key = $ua->{'product'}." ".$ua->{'version'};
        $h->{$key}+=$cnt;
    }
    $histodata=$h;
}

#Dump results
foreach my $ua (sort {$a<=>$b}keys %{$histodata}){
    my $cnt = $histodata->{$ua};
    if ($freq){
        print "$ua $cnt\n";
    }else{
        print "$ua\n";
    }
}

exit 0;
