#!/usr/bin/perl
package FuzzyCVE;
use strict;
use Data::Dumper;
#Take a keyword list as input; in order to avoid to go through the bulk data
#again and again
sub new{
    my $type = shift;
    my $self={};
    $self->{keyword_list}=\@_;
    # State variables
    $self->{entry}= undef;
    $self->{newEntry} = 0;
    $self->{keywordCache}={};
    bless $self,$type;
    return $self;
}

sub readln{
    my ($self,$line) = @_;
    # Compress spaces
    #$line=~s/\s+/ /g;
    if ($line=~/^====/){
        $self->{newEntry} = 1;
        if (defined($self->{entry})){
            my $out ="";
            my ($name,$status,$phase,$category,$msg) = 
               $self->process_entry($self->{entry});
            $self->add_to_cache($msg,$name);
            #my ($version, $before) = $self->process_msg($msg,$name);
            # Reset entry
            $self->{entry}="";
        }
        next;
     }
     $self->{entry}.=$line;
}

sub load{
    my ($self,$filename) =@_;
    if (defined($filename)){
        open F,"<$filename" or die "$?\n";
        while (<F>){
            $self->readln($_);
        }
        close F;
    }else{
    # Read from stdin
        while(<>){
            chomp;
            $self->readln($_);
        }
    }
}

sub process_msg{
    my ($self,$msg,$name,$keyword) = @_;
    my ($version, $before)=(undef,0);
    if ($msg=~/$keyword ([0-9.]+) /i){
        $version = $1;
    }
    # Handle keyword earlier
    if ($msg=~/$keyword ([0-9.]+ )and earlier/i){
        $before = 1;
    }
    
    if ($msg=~/$keyword before version ([0-9.]+ )/i){
        $before = 1;
    }
    return ($version,$before);
}


sub process_entry{
    my ($self,$entry)=@_;
    my ($name,$status,$category, $phase,$msg);
    my $inComments = 0;
    my @rows = split('\n',$entry);
    # Take only records that start with name:
    if (@rows[0]=~/^Name:(.+)/){
        $name = $1;
        $name=~s/\s//g;
        shift @rows;
        foreach my $line (@rows){
            if ($line=~/^Status:(.+)/){
                $status=$1;
                $status=~s/\s//g;
                next;
            }
            # Skip references
            if ($line=~/^Reference:/){
                next;
            }
            # Skip URL
            if ($line=~/^URL:/){
                next;
            }
            # Skip empty lines
            if ($line eq ""){
                next;
            }
            if ($line=~/^Category:(.+)/){
                $category = $1;
                $category=~s/\s//g;
                next;
            }
            if ($line=~/^Phase:(.+)/){
                $phase= $1;
                $phase=~s/\s//g;
                next;
            }
            # Skip comments
            if ($line=~/^Current Votes:/){
                return ($name,$status,$phase,$category,$msg);
            }    
            # The rest is considered as free text message
            $msg.=$line."\n";
        }
        return ($name,$status,$phase,$category,$msg);
       
    }
}

sub add_to_cache{
    my ($self,$msg,$name) = @_;
    # Handle only CVE entries with a valid name
    if (!defined($name)){
        return;
    }
    # Go through the keyword list and put all matches in the cache
    foreach my $kw (@{$self->{keyword_list}}){
        my ($version, $before) = $self->process_msg($msg,$name,$kw);
        # Was a lookup found?
        if (!defined($version)){
            next;
        }
        #FIXME Currently ignore before tag
        if (!exists($self->{keywordCache}->{$kw})){
            $self->{keywordCache}->{$kw}={};
        }
        if (!exists($self->{keywordCache}->{$kw}->{$version})){
            $self->{keywordCache}->{$kw}->{$version}=();
        } 
        push(@{$self->{keywordCache}->{$kw}->{$version}},$name);
    }
}

sub get_product_cve_num{
    my ($self,$product,$version) = @_;
    if (defined($self->{'keywordCache'}->{$product})){
        # Do a version lookup
        if (defined($version)){
            if (defined($self->{keywordCache}->{$product}->{$version})){
                return scalar(@{$self->{keywordCache}->{$product}->{$version}});
            }else{
                return 0;
            }
        }
        # Return all the vulnerabilities number for this product
        my $sum = 0;
        foreach my $ver (keys %{$self->{keywordCache}->{$product}}){
            $sum+=scalar(@{$self->{keywordCache}->{$product}->{$ver}});
        }
        return $sum;
    }
    # Did not see vulnerability 
    return 0;
}
1;
