#!/usr/bin/perl
use strict;
use Getopt::Long;
use Data::Dumper;
use FuzzyCVE;

sub usage{
    my $code = shift;
    print <<END;
fuzzy cve lookup [-h] [-k] [-v] 

CVE records are extracted. CVE name,status,phase and categories are parsed.
Unfortunately the CVE entry is a free text field. Thus this script only 
matches keyword followed by a version ([0-9.]). If the patterns

    "keyword before version" 
    "and earlier" 

are found the field "before" is set. Thus all previous versions should
be taken into account for later processing. 
 
    -i Specifies the CVE-list input file
    -k Specifies a list of keywords that should be lookup (file one product name per
                                                           line)
    -m Match the version given by this option
    -s Serialize FuzzyCVE object that it can be used several times; Generating 
       a FuzzyCVE cache take a long time ...

Currently only the number of CVE entries for a product are displayed

AUTHOR:
     Gerard Wagener

LICENSE
    GPL
END
exit $code;
}

#### Main ###
my $isHelp =0;
my $isVersion = 0;
my $inputFile = undef;
my $keywordFile=undef;
my $matchVersion = undef;
my $serializeFile = undef;

GetOptions("help"=>\$isHelp, "version"=>\$isVersion, "keyword=s"=>\$keywordFile,
           "match=s"=>\$matchVersion,"inputfile=s"=>\$inputFile,
           "serialize=s"=>\$serializeFile) or die "";


if ($isHelp){
    &usage(0);
    exit 0;
}

if (!defined($inputFile) or (! -e $inputFile)){
    &usage(1);
}

if (!defined($keywordFile) or (! -e $keywordFile)){
    &usage(1);
}

# Load keyword file
open F,"<$keywordFile" or die "Cannot open keyword file $?\n";
my @kw = ();
while (<F>){
    chomp;
    push(@kw,$_);
}
close F;

my $fc = FuzzyCVE->new(@kw);
$fc->load($inputFile);


if (defined($serializeFile)){
    open F,">$serializeFile" or die "Cannot open serialize file\n";
    print F Dumper($fc);
    close F;
    exit 0;
}

# Return all the vulnerabilies for a the keywords
for my $k (@kw){
    my $n = $fc->get_product_cve_num($k);
    print "$k: $n\n"; 
}
